;;;;;;;;;;;;;;;;;;;;;;;;;;;;;<--OGI-->;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                        ;;
;;             Center for Spoken Language Understanding                   ;;
;;        Oregon Graduate Institute of Science & Technology               ;;
;;                         Portland, OR USA                               ;;
;;                        Copyright (c) 2000                              ;;
;;                                                                        ;;
;;      This module is not part of the CSTR/University of Edinburgh       ;;
;;               release of the Festival TTS system.                      ;;
;;                                                                        ;;
;;  In addition to any conditions disclaimers below, please see the file  ;;
;;  "license_ogi_tts.txt" distributed with this software for information  ;;
;;  on usage and redistribution, and for a DISCLAIMER OF ALL WARRANTIES.  ;;
;;                                                                        ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;<--OGI-->;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(set! ogi_debug_mwm5_unitsel nil)

(define (item.alloname seg) 
  (let ((name (item.name seg)))
  (if (string-equal (item.feat seg "allophone_name") "dx") 
    "dx"
    name)))

(define (mwm5_OGI_di_unitsel utt)
  "Replace unit selection method defined in ogi_unitsel.scm.
   Strategy: if vowel or glide, determine syllable boundary, 
   check if special case diphone exists" 
  (let ((SelectedUnits) (seglist (utt.relation.items utt "Segment"))
	segL 
	segR
	Lnm 
	Rnm
	diphL
	diphR
	segRR
	RRnm 
	nextliq
	nextcoda 
	uvplos
	onset
	segLL 
	s
	coda
	prevvowel 
	plos 
	segLL 
	cand 
	(prevdiphL "pau"))

    (mapcar
     (lambda (segL)
       ;;(format stderr "SEGMENT left: %s\n" (item.alloname segL))
       ;; (format stderr "SEGMENT onset: %s\n" (item.feat segL 'ogi_onsetcoda))

       (set! segR (item.next segL))
       (if (equal? segR nil)
	   nil    ;; if last segment of utt, don't do anything
	   ;; else 
	   (begin
	     (set! Lnm (item.alloname segL))
	     (set! Rnm (item.alloname segR))
	     (set! diphL Lnm)
	     (set! diphR Rnm)
	     (cond 
	      
	      ;; COND 1 -> C-V  if right phone is vowel, check if followed by liquid or preceded by plosive
	      ((string-equal (item.feat segR "ph_vc") "+")
	       
	       ;; check if left phone is unvoiced plosive in onset, not preceded by s
	       ;; (aspirated version or not)
	       (set! uvplos (or (string-equal Lnm "p") 
				(string-equal Lnm "t") 
				(string-equal Lnm "k")
				
				(string-equal Lnm "b")
				(string-equal Lnm "d")
				(string-equal Lnm "g")))
	       
	       (set! liquid (or (string-equal Lnm "9r") 
				(string-equal Lnm "l")))
	       
	       (set! onset (string-equal (item.feat segL 'ogi_onsetcoda ) "onset"))
	       (set! segLL (item.prev segL))
	       (set! s (and segLL (string-equal (item.alloname segLL) 's) 
			    (string-equal (item.feat segLL 'ogi_onsetcoda ) "onset")))
	       (set! stress (string-equal (item.feat segR "R:SylStructure.parent.stress") "1"))
	       (set! prev_is_vowel (and segLL (item.feat segLL "ph_vc") "+"))
	       
	       ;; [ptkbdg]-V  becomes [ptkbdg].-V if not onset
	       (if (and uvplos 
			(not onset))
		   (set! diphL (string-append Lnm "."))
		   )
	       
	       ;; rename diphL for example to "9r."
	       (if (and liquid
			;(not prev_is_vowel)  ; VP check this because "onset" is screwed up by festival
			(not onset))
		   (set! diphL (string-append Lnm "." )))
	       
	       ;; check if right phone is followed by liq in syllable coda
	       ;; then special liquid allophone for the vowel  C-V-liq
	       (set! segRR (item.next segR))

	       (if (and segRR (not (string-equal (item.feat segL "ph_vc") "+")))
		   (begin
		     (set! RRnm (item.alloname segRR))
		     (set! nextliq (or (string-equal RRnm "l") (string-equal RRnm "9r")))
		     (set! nextcoda (string-equal (item.feat segRR 'ogi_onsetcoda ) "coda"))

		     ;; VP -> because Syllable screwed up by Festival
		     ;(set! segRRR (item.next segRR))
		     ;(set! segRRR_vowel (and segRRR  (string-equal (item.feat segRRR "ph_vc") "+")))
		     ;(print "decide to put" segRRR)
		     
		     (if (and nextliq
			      nextcoda
					; (not segRRR_vowel)
			      )
			 (begin
			   (set! diphL Lnm)  ; Remove any previously set "." in the end of the consonant
			
			   ;; map E-9r to @-9r
			   (if (and (string-equal Rnm "E") (string-equal RRnm "9r")) 
			       (set! Rnm "@"))

			   ;; map A-l to >-l
			   (if (and (string-equal Rnm "A") (string-equal RRnm "l")) 
			       (set! Rnm ">"))

			   ;; map U-9r to u-9r
			   (if (and (string-equal Rnm "U") (string-equal RRnm "9r")) 
			       (set! Rnm "u"))
			   
			   ;; RENAME diphR for example to "i:+k+9r."
			   (set! diphR (string-append Rnm "+" diphL "+" RRnm "."))))))
	       )
	      
	      ;; COND
	      ;; if right phone is liquid, check if preceded by vowel or plosive
	      ((or (string-equal Rnm "l") (string-equal Rnm "9r"))
	       (set! coda (string-equal (item.feat segR 'ogi_onsetcoda ) "coda"))
	       (set! prevvowel (string-equal (item.feat segL "ph_vc") "+"))
	       
	       (if (and coda prevvowel) 
		   (begin
		     ;; map E to @
		     (if (and (string-equal Lnm "E") (string-equal Rnm "9r")) 
			 (set! Lnm "@"))

		     ;; map A-l to >-l
		     (if (and (string-equal Lnm "A") (string-equal Rnm "l")) 
			 (set! Lnm ">"))

		     ;; map U-9r to u-9r 
		     (if (and (string-equal Lnm "U") (string-equal Rnm "9r")) 
			 (set! Lnm "u"))

		     ;; RENAME diphL for example to "i:+p+l."; this needs to match previously selected .p-i:l. 
		     (set! diphL (string-append Lnm "+" prevdiphL "+" Rnm "." ))
		     (set! diphR (string-append Rnm "."))))
	       
	       (set! plos (member Lnm '("p" "t" "k" "b" "d" "g")))
	       (set! onset (string-equal (item.feat segL 'ogi_onsetcoda ) "onset"))
	       (set! segLL (item.prev segL))
	       (set! s (and segLL (string-equal (item.alloname segLL) 's) 
			    (string-equal (item.feat segLL 'ogi_onsetcoda ) "onset")))
	       (set! stress (string-equal (item.feat segR "R:SylStructure.parent.stress") "1"))
	      
	       (if (and plos onset (not s) stress)
		   ;; rename diphL for example to ".b"
		   (set! diphL (string-append "." Lnm))))
	      )
	     ;; END OF COND

	     (set! cand (string-append diphL "-" diphR))
	     (if ogi_debug_mwm5_unitsel	    
		 (format stderr " diphL-diphL= %s-%s\n" diphL diphR))

	     (set! prevdiphL diphL)
	     
	     ;; look up in dbase
	     (set! indx (OGIdbase.unitsel_name cand))
	     
	     (if (equal? indx nil)
		 (if ogi_debug_mwm5_unitsel	  
		     (format stderr "FAILED......................... BACK TO %s %s\n" Lnm Rnm)
		     )
		 )

	     ;; look for alternates if original not found
	     (if (equal? indx nil)
		 (set! indx (OGI_di_alt Lnm Rnm)))
	     
	     ;; lookup info structure for this unit
	     (set! sel (list (OGIdbase.lookup_info indx)))
	     
	     ;; append info to list
	     (set! SelectedUnits (append SelectedUnits sel))
	     )))
     seglist)
    SelectedUnits
    )
)
